@@ -1,38 +1,31 @@
-import concurrent.futures
+import asyncio
 import multiprocessing
-import os
-import threading
 from abc import abstractmethod
 from datetime import datetime
 from typing import TYPE_CHECKING, ClassVar, Iterator, Tuple, Type
 
+from fsspec.asyn import get_loop
 from tqdm import tqdm
 
 from dql.client.base import Bucket, Client
 from dql.data_storage import AbstractDataStorage
-from dql.listing import Listing
 from dql.nodes_fetcher import NodesFetcher
 from dql.nodes_thread_pool import NodeChunk
 
 if TYPE_CHECKING:
     from fsspec.spec import AbstractFileSystem
 
+FETCH_WORKERS = 100
+
 
 class FSSpecClient(Client):
-    THREAD_LOCK = threading.Lock()
     MAX_THREADS = multiprocessing.cpu_count()
-    MAX_THREADS_LISTING = int(
-        os.environ.get("MAX_THREADS_LISTING", MAX_THREADS * 10)
-    )
-    CHUNK_SIZE = 1024 * 1024
     FS_CLASS: ClassVar[Type["AbstractFileSystem"]]
     PREFIX: ClassVar[str]
 
     def __init__(self, name, fs):
         self.name = name
         self.fs = fs
-        self.lock = threading.Lock()
-        self.global_counter = 0
 
     @classmethod
     def create_fs(cls, **kwargs) -> "AbstractFileSystem":
@@ -73,92 +66,67 @@ class FSSpecClient(Client):
         path_split = fill_path.split("/", 1)
         storage = path_split[0]
         path = path_split[1] if len(path_split) > 1 else ""
-        return cls(storage, **kwargs), path
-
-    def fetch(self, listing):
-        def initializer_worker(local, data_storage):
-            """
-            Adding cloned instance of data storage as thread local for each
-            thread in thread pool so that we don't need to clone it for each
-            task
-            """
-            local.data_storage = data_storage.clone()
+        client = cls(storage, **kwargs)
+        return client, path
 
+    async def fetch(self, listing):
         root_id = listing.insert_root()
+        progress_bar = tqdm(desc=f"Listing {self.uri}", unit=" objects")
+        loop = get_loop()
 
-        local = threading.local()
-        # setting up data_storage for main thread
-        local.data_storage = listing.data_storage
+        queue = asyncio.Queue()
+        queue.put_nowait((root_id, ""))
 
-        with concurrent.futures.ThreadPoolExecutor(
-            self.MAX_THREADS_LISTING,
-            initializer=initializer_worker,
-            initargs=(
-                local,
-                listing.data_storage,
-            ),
-        ) as executor:
-            progress_bar = tqdm(desc=f"Listing {self.uri}", unit=" objects")
-            tasks = self._fetch_dir(
-                root_id,
-                "",
-                "/",
-                executor,
-                progress_bar,
-                local,
-            )
-            while tasks:
-                for task in concurrent.futures.as_completed(tasks):
-                    tasks.remove(task)
-                    tasks.update(task.result())
+        async def worker(queue):
+            while True:
+                dir_id, prefix = await queue.get()
+                try:
+                    subdirs = await self._fetch_dir(
+                        dir_id, prefix, "/", progress_bar, listing
+                    )
+                    for subdir in subdirs:
+                        queue.put_nowait(subdir)
+                finally:
+                    queue.task_done()
 
-    def _fetch_dir(self, dir_id, prefix, delimiter, executor, pbar, local):
-        data_storage = local.data_storage
+        workers = []
+        for _ in range(FETCH_WORKERS):
+            workers.append(loop.create_task(worker(queue)))
 
+        await queue.join()
+        for worker in workers:
+            worker.cancel()
+        await asyncio.gather(*workers)
+
+    async def _fetch_dir(self, dir_id, prefix, delimiter, pbar, listing):
         path = f"{self.name}/{prefix}"
-        infos = self.fs.ls(path, detail=True, versions=True)
+        # pylint:disable-next=protected-access
+        infos = await self.fs._ls(path, detail=True, versions=True)
         files = []
-        tasks = set()
+        subdirs = set()
         for info in infos:
             full_path = info["name"]
             _, subprefix, _ = self.fs.split_path(info["name"])
             if info["type"] == "directory":
                 name = full_path.split(delimiter)[-1]
-                new_dir_id = (
-                    Listing._insert_dir(  # pylint:disable=protected-access
-                        data_storage, dir_id, name, datetime.max, subprefix
-                    )
+                new_dir_id = listing.insert_dir(
+                    dir_id, name, datetime.max, subprefix
                 )
-
-                task = executor.submit(
-                    self._fetch_dir,
-                    new_dir_id,
-                    subprefix,
-                    delimiter,
-                    executor,
-                    pbar,
-                    local,
-                )
-                tasks.add(task)
+                subdirs.add((new_dir_id, subprefix))
             else:
                 files.append(
                     self._df_from_info(info, dir_id, delimiter, subprefix)
                 )
         if files:
-            data_storage.insert(files, is_dir=False)
-        self.update_pbar_threadsafe(pbar, len(infos))
-        return tasks
+            listing.data_storage.insert(files, is_dir=False)
+        pbar.update(len(subdirs) + len(files))
+        return subdirs
 
     @classmethod
     @abstractmethod
     def _df_from_info(cls, v, dir_id, delimiter, path):
         ...
 
-    @classmethod
-    def update_pbar_threadsafe(cls, pbar, obj_num):
-        with cls.THREAD_LOCK:
-            pbar.update(obj_num)
-
     def fetch_nodes(
         self,
         file_path,
