@@ -4,6 +4,10 @@ from gcsfs import GCSFileSystem
 
 from .fsspec import FSSpecClient
 
+# Patch gcsfs for consistency with s3fs
+# pylint:disable-next=protected-access
+GCSFileSystem.set_session = GCSFileSystem._set_session
+
 
 class GCSClient(FSSpecClient):
     FS_CLASS = GCSFileSystem
@@ -13,10 +17,6 @@ class GCSClient(FSSpecClient):
     def __init__(self, name, **kwargs):
         super().__init__(name, self.create_fs(**kwargs))
 
-    @property
-    def uri(self):
-        return f"gcs://{self.name}"
-
     @classmethod
     def _df_from_info(cls, v, dir_id, delimiter, path):
         return {
