@@ -1,9 +1,12 @@
+from datetime import datetime
 from typing import cast
 
-from s3fs import S3FileSystem
+from dql.vendored.s3fs import S3FileSystem  # type: ignore[attr-defined]
 
 from .fsspec import FSSpecClient
 
+FETCH_CHUNKSIZE = 1000
+
 
 class ClientS3(FSSpecClient):
     FS_CLASS = S3FileSystem
@@ -30,9 +33,36 @@ class ClientS3(FSSpecClient):
 
         return cast(S3FileSystem, super().create_fs(**kwargs))
 
-    @property
-    def uri(self):
-        return f"s3://{self.name}"
+    async def _fetch_dir(self, dir_id, prefix, delimiter, pbar, listing):
+        if prefix:
+            prefix = prefix.lstrip("/") + "/"
+        files = []
+        subdirs = set()
+        # pylint:disable-next=protected-access
+        async for info in self.fs._iterdir(
+            self.name, prefix=prefix, versions=True
+        ):
+            full_path = info["name"]
+            _, subprefix, _ = self.fs.split_path(info["name"])
+            if info["type"] == "directory":
+                name = full_path.split(delimiter)[-1]
+                new_dir_id = listing.insert_dir(
+                    dir_id, name, datetime.max, subprefix
+                )
+                subdirs.add((new_dir_id, subprefix))
+            else:
+                files.append(
+                    self._df_from_info(info, dir_id, delimiter, subprefix)
+                )
+                if len(files) >= FETCH_CHUNKSIZE:
+                    listing.data_storage.insert(files, is_dir=False)
+                    pbar.update(len(files))
+                    files = []
+        if files:
+            listing.data_storage.insert(files, is_dir=False)
+            pbar.update(len(files))
+        pbar.update(len(subdirs))
+        return subdirs
 
     @staticmethod
     def clean_s3_version(ver):
