@@ -1,5 +1,6 @@
 import logging
 from abc import ABC, abstractmethod
+from datetime import datetime
 from typing import Any, Dict, Iterable, Iterator, List, Optional, Tuple
 
 from dql.node import Node, NodeWithPath
@@ -46,7 +47,13 @@ class AbstractDataStorage(ABC):
         """
 
     @abstractmethod
-    def mark_storage_indexed(self, storage: Storage, ttl: int) -> None:
+    def mark_storage_indexed(
+        self,
+        storage_id: str,
+        status: int,
+        ttl: int,
+        end_time: Optional[datetime] = None,
+    ) -> None:
         """
         Marks storage as indexed.
         This method should be called when index operation is finished
