@@ -12,10 +12,7 @@ from dql.error import DQLError
 from dql.jmes_sql.transpiler import Transpiler
 from dql.node import Node, NodeWithPath
 from dql.storage import Storage
-from dql.utils import DQL_PATH, GLOB_CHARS, sql_file_filter
-
-DB_FILE_NAME = "db"
-
+from dql.utils import GLOB_CHARS, DQLDir, sql_file_filter
 
 logger = logging.getLogger("dql")
 
@@ -31,16 +28,10 @@ class DefaultDataStorage(AbstractDataStorage):
 
     def __init__(self, db_file=None, table_name: str = ""):
         self.table_name = table_name
-        self.db_file = db_file
-        if not db_file:
-            if not os.path.isdir(DQL_PATH):
-                print("Creating '.dql/' directory")
-                os.mkdir(DQL_PATH)
-            db_file = os.path.join(DQL_PATH, DB_FILE_NAME)
-
+        self.db_file = db_file if db_file else DQLDir.find().db
         try:
             self.db = duckdb.connect(  # pylint:disable=c-extension-no-member
-                database=db_file,
+                database=self.db_file,
                 read_only=False,
             )
             self._init_storage_table()
@@ -57,7 +48,8 @@ class DefaultDataStorage(AbstractDataStorage):
                 type        VARCHAR,
                 name        VARCHAR,
                 timestamp   DATETIME,
-                expires     DATETIME
+                expires     DATETIME,
+                status      UINTEGER NOT NULL,
             )
         """
         )
@@ -210,15 +202,6 @@ class DefaultDataStorage(AbstractDataStorage):
                 anno            JSON
             );
             CREATE SEQUENCE {seq_name} START 1;
-
-            CREATE TABLE IF NOT EXISTS buckets
-            (
-                id          VARCHAR,
-                type        VARCHAR,
-                name        VARCHAR,
-                timestamp   DATETIME,
-                expires     DATETIME
-            )
         """
         )
 
@@ -293,7 +276,12 @@ class DefaultDataStorage(AbstractDataStorage):
         return 0
 
     def get_storage_all(self) -> Iterator[Storage]:
-        for values in self.db.execute("SELECT * FROM buckets").fetchall():
+        for values in self.db.execute(
+            """
+            SELECT id, type, name, timestamp, expires, status
+            FROM buckets
+            """
+        ).fetchall():
             yield Storage(*values)
 
     def get_storage(
@@ -301,7 +289,8 @@ class DefaultDataStorage(AbstractDataStorage):
     ) -> Optional[Storage]:
         res = self.db.execute(
             """
-            SELECT * FROM buckets
+            SELECT id, type, name, timestamp, expires, status
+            FROM buckets
             WHERE id = ? AND expires > ?
             """,
             [storage_id, expires_at],
@@ -370,7 +359,7 @@ class DefaultDataStorage(AbstractDataStorage):
         self.db.execute(
             f"""
             DELETE FROM buckets WHERE id = '{storage.id}';
-            INSERT INTO buckets VALUES(?, ?, ?, ?, ?)
+            INSERT INTO buckets VALUES(?, ?, ?, ?, ?, ?)
             """,
             [
                 storage.id,
@@ -378,6 +367,7 @@ class DefaultDataStorage(AbstractDataStorage):
                 storage.name,
                 old_storage.timestamp if old_storage else None,
                 old_storage.expires if old_storage else None,
+                storage.status,
             ],
         )
 
@@ -396,18 +386,26 @@ class DefaultDataStorage(AbstractDataStorage):
             ],
         )
 
-    def mark_storage_indexed(self, storage: Storage, ttl: int) -> None:
-        timestamp = datetime.now(timezone.utc)
+    def mark_storage_indexed(
+        self,
+        storage_id: str,
+        status: int,
+        ttl: int,
+        end_time: Optional[datetime] = None,
+    ) -> None:
+        if end_time is None:
+            end_time = datetime.now(timezone.utc)
         self.db.execute(
             """
             UPDATE buckets
-            SET timestamp = ?, expires = ?
+            SET timestamp = ?, expires = ?, status = ?
             WHERE id = ?
             """,
             [
-                timestamp,
-                Storage.get_expiration_time(timestamp, ttl),
-                storage.id,
+                end_time,
+                Storage.get_expiration_time(end_time, ttl),
+                status,
+                storage_id,
             ],
         )
 
