@@ -5,6 +5,7 @@ from shutil import copy2
 from typing import DefaultDict, Iterable, List
 
 import yaml
+from fsspec.asyn import get_loop, sync
 from reflink import reflink
 from reflink.error import ReflinkImpossibleError
 from tqdm import tqdm
@@ -41,7 +42,7 @@ class Listing:
         return self.storage.id
 
     def fetch(self):
-        self.client.fetch(self)  # pylint:disable=protected-access
+        sync(get_loop(), self.client.fetch, self)
 
     @staticmethod
     def _insert_dir(data_storage, parent_id, name, time, path):
